import re
import xml.etree.ElementTree as ET
from dataclasses import field
from typing import Any, Optional

import flet as ft

_PLOTLY_IMPORT_ERROR: Optional[ImportError] = None

try:
    from plotly.graph_objects import Figure
except ImportError as e:  # pragma: no cover - depends on optional dependency
    Figure = Any  # type: ignore[assignment]
    _PLOTLY_IMPORT_ERROR = e

__all__ = ["PlotlyChart"]


def _require_plotly() -> None:
    if _PLOTLY_IMPORT_ERROR is not None:
        raise ModuleNotFoundError(
            'Install "plotly" Python package to use PlotlyChart control.'
        ) from _PLOTLY_IMPORT_ERROR


@ft.control(kw_only=True)
class PlotlyChart(ft.Container):
    """
    Displays a [Plotly](https://plotly.com/python/) chart.

    Warning:
        This control requires the [`plotly`](https://plotly.com/python/) and
        [`kaleido`](https://github.com/plotly/Kaleido) Python
        packages to be installed.

        See this [installation guide](index.md#installation) for more information.

    ```python
    import plotly.express as px

    data_canada = px.data.gapminder().query("country == 'Canada'")
    fig = px.bar(data_canada, x="year", y="pop")
    fch.PlotlyChart(figure=fig, expand=True)
    ```

    """

    figure: Figure = field(metadata={"skip": True})
    """
    Plotly figure to draw.

    The value is an instance of [`plotly.graph_objects.Figure`](https://plotly.com/python-api-reference/generated/plotly.graph_objects.Figure.html).
    """

    original_size: bool = False
    """
    Whether to display this chart in original size.

    Set to `False` for it to fit it's configured bounds.
    """

    def init(self):
        _require_plotly()
        self.alignment = ft.Alignment.CENTER
        self.__img = ft.Image(src="", fit=ft.BoxFit.FILL)
        self.content = self.__img

    def before_update(self):
        super().before_update()
        if self.figure is not None:
            svg = self.figure.to_image(format="svg").decode("utf-8")

            if not self.original_size:
                root = ET.fromstring(svg)
                w = float(re.findall(r"\d+", root.attrib["width"])[0])
                h = float(re.findall(r"\d+", root.attrib["height"])[0])
                self.__img.aspect_ratio = w / h
            self.__img.src = svg



from dataclasses import dataclass, field
from typing import Optional, Union

import flet as ft
from flet_charts.types import ChartDataPointTooltip

__all__ = ["CandlestickChartSpot", "CandlestickChartSpotTooltip"]


@dataclass
class CandlestickChartSpotTooltip(ChartDataPointTooltip):
    """Tooltip configuration for the [`CandlestickChartSpot`][(p).]."""

    bottom_margin: ft.Number = 8
    """
    Space between the tooltip bubble and the candlestick.
    """

    def copy(
        self,
        *,
        text: Optional[str] = None,
        text_style: Optional[ft.TextStyle] = None,
        text_align: Optional[ft.TextAlign] = None,
        text_spans: Optional[list[ft.TextSpan]] = None,
        rtl: Optional[bool] = None,
        bottom_margin: Optional[ft.Number] = None,
    ) -> "CandlestickChartSpotTooltip":
        """
        Returns a copy of this object with the specified properties overridden.
        """
        return CandlestickChartSpotTooltip(
            text=text if text is not None else self.text,
            text_style=text_style if text_style is not None else self.text_style,
            text_align=text_align if text_align is not None else self.text_align,
            text_spans=text_spans.copy()
            if text_spans is not None
            else (self.text_spans.copy() if self.text_spans is not None else None),
            rtl=rtl if rtl is not None else self.rtl,
            bottom_margin=bottom_margin
            if bottom_margin is not None
            else self.bottom_margin,
        )


@ft.control("CandlestickChartSpot")
class CandlestickChartSpot(ft.BaseControl):
    """Represents a candlestick rendered on a [`CandlestickChart`][(p).]."""

    x: ft.Number
    """
    The position of the candlestick on the X axis.
    """

    open: ft.Number
    """
    The open value of the candlestick.
    """

    high: ft.Number
    """
    The high value of the candlestick.
    """

    low: ft.Number
    """
    The low value of the candlestick.
    """

    close: ft.Number
    """
    The close value of the candlestick.
    """

    selected: bool = False
    """
    Whether to treat this candlestick as selected.
    """

    tooltip: Union[CandlestickChartSpotTooltip, str] = field(
        default_factory=lambda: CandlestickChartSpotTooltip()
    )
    """
    Tooltip configuration for this candlestick.
    """

    show_tooltip: bool = True
    """
    Whether the tooltip should be shown when this candlestick is highlighted.
    """

    def before_update(self):
        super().before_update()
        self._internals["tooltip"] = (
            CandlestickChartSpotTooltip(text=self.tooltip)
            if isinstance(self.tooltip, str)
            else self.tooltip
        )



from dataclasses import dataclass, field
from typing import Any, Optional

import flet as ft
from flet_charts.chart_axis import ChartAxis
from flet_charts.scatter_chart_spot import ScatterChartSpot
from flet_charts.types import ChartEventType, ChartGridLines, HorizontalAlignment

__all__ = ["ScatterChart", "ScatterChartEvent", "ScatterChartTooltip"]


@dataclass
class ScatterChartTooltip:
    """Configuration of the tooltip for [`ScatterChart`][(p).]s."""

    bgcolor: ft.ColorValue = "#FF607D8B"
    """
    The tooltip's background color.
    """

    border_radius: Optional[ft.BorderRadiusValue] = None
    """
    The tooltip's border radius.
    """

    padding: ft.PaddingValue = field(
        default_factory=lambda: ft.Padding.symmetric(vertical=8, horizontal=16)
    )
    """
    Applies a padding for showing contents inside the tooltip.
    """

    max_width: ft.Number = 120
    """
    Restricts the tooltip's width.
    """

    rotation: ft.Number = 0.0
    """
    The tooltip's rotation angle in degrees.
    """

    horizontal_offset: ft.Number = 0
    """
    Applies horizontal offset for showing tooltip.
    """

    horizontal_alignment: HorizontalAlignment = HorizontalAlignment.CENTER
    """
    The tooltip's horizontal alignment.
    """

    border_side: ft.BorderSide = field(default_factory=lambda: ft.BorderSide.none())
    """
    The tooltip's border side.
    """

    fit_inside_horizontally: bool = False
    """
    Forces the tooltip to shift horizontally inside the chart, if overflow happens.
    """

    fit_inside_vertically: bool = False
    """
    Forces the tooltip to shift vertically inside the chart, if overflow happens.
    """

    def copy(
        self,
        *,
        bgcolor: Optional[ft.ColorValue] = None,
        border_radius: Optional[ft.BorderRadiusValue] = None,
        padding: Optional[ft.PaddingValue] = None,
        max_width: Optional[ft.Number] = None,
        rotation: Optional[ft.Number] = None,
        horizontal_offset: Optional[ft.Number] = None,
        horizontal_alignment: Optional[HorizontalAlignment] = None,
        border_side: Optional[ft.BorderSide] = None,
        fit_inside_horizontally: Optional[bool] = None,
        fit_inside_vertically: Optional[bool] = None,
    ) -> "ScatterChartTooltip":
        """
        Returns a copy of this object with the specified properties overridden.
        """
        return ScatterChartTooltip(
            bgcolor=bgcolor if bgcolor is not None else self.bgcolor,
            border_radius=(
                border_radius if border_radius is not None else self.border_radius
            ),
            padding=padding if padding is not None else self.padding,
            max_width=max_width if max_width is not None else self.max_width,
            rotation=rotation if rotation is not None else self.rotation,
            horizontal_offset=(
                horizontal_offset
                if horizontal_offset is not None
                else self.horizontal_offset
            ),
            horizontal_alignment=(
                horizontal_alignment
                if horizontal_alignment is not None
                else self.horizontal_alignment
            ),
            border_side=border_side if border_side is not None else self.border_side,
            fit_inside_horizontally=(
                fit_inside_horizontally
                if fit_inside_horizontally is not None
                else self.fit_inside_horizontally
            ),
            fit_inside_vertically=(
                fit_inside_vertically
                if fit_inside_vertically is not None
                else self.fit_inside_vertically
            ),
        )


@dataclass
class ScatterChartEvent(ft.Event["ScatterChart"]):
    type: ChartEventType
    """
    The type of the event that occurred.
    """

    spot_index: Optional[int] = None
    """
    The index of the touched spot, if any.
    """


@ft.control("ScatterChart")
class ScatterChart(ft.LayoutControl):
    """
    A scatter chart control.

    ScatterChart draws some points in a square space,
    points are defined by [`ScatterChartSpot`][(p).]s.

    ```python

    fch.ScatterChart(
        aspect_ratio=1.0,
        spots=[fch.ScatterChartSpot(
            x=random.uniform(4, 50),
            y=random.uniform(4, 50),
        )
        for _ in range(30)]
    )
    ```

    """

    spots: list[ScatterChartSpot] = field(default_factory=list)
    """
    List of [`ScatterChartSpot`][(p).]s to show on the chart.
    """

    animation: ft.AnimationValue = field(
        default_factory=lambda: ft.Animation(
            duration=ft.Duration(milliseconds=150), curve=ft.AnimationCurve.LINEAR
        )
    )
    """
    Controls chart implicit animation.
    """

    interactive: bool = True
    """
    Enables automatic tooltips when hovering chart bars.
    """

    long_press_duration: Optional[ft.DurationValue] = None
    """
    The duration of a long press on the chart.
    """

    bgcolor: Optional[ft.ColorValue] = None
    """
    The chart's background color.
    """

    border: Optional[ft.Border] = None
    """
    The border around the chart.
    """

    horizontal_grid_lines: Optional[ChartGridLines] = None
    """
    Controls drawing of chart's horizontal lines.
    """

    vertical_grid_lines: Optional[ChartGridLines] = None
    """
    Controls drawing of chart's vertical lines.
    """

    left_axis: Optional[ChartAxis] = None
    """
    Configures the appearance of the left axis, its title and labels.
    """

    top_axis: Optional[ChartAxis] = None
    """
    Configures the appearance of the top axis, its title and labels.
    """

    right_axis: Optional[ChartAxis] = None
    """
    Configures the appearance of the right axis, its title and labels.
    """

    bottom_axis: Optional[ChartAxis] = None
    """
    Configures the appearance of the bottom axis, its title and labels.
    """

    baseline_x: Optional[ft.Number] = None
    """
    The baseline value for X axis.
    """

    min_x: Optional[ft.Number] = None
    """
    The minimum displayed value for X axis.
    """

    max_x: Optional[ft.Number] = None
    """
    The maximum displayed value for X axis.
    """

    baseline_y: Optional[ft.Number] = None
    """
    Baseline value for Y axis.
    """

    min_y: Optional[ft.Number] = None
    """
    The minimum displayed value for Y axis.
    """

    max_y: Optional[ft.Number] = None
    """
    The maximum displayed value for Y axis.
    """

    tooltip: ScatterChartTooltip = field(default_factory=lambda: ScatterChartTooltip())
    """
    The tooltip configuration for the chart.
    """

    show_tooltips_for_selected_spots_only: bool = False
    """
    Whether to permanently and only show the tooltips of spots with their
    [`selected`][(p).ScatterChartSpot.selected] property set to `True`.
    """

    rotation_quarter_turns: ft.Number = 0
    """
    Number of quarter turns (90-degree increments) to rotate the chart.
    Ex: `1` rotates the chart `90` degrees clockwise,
    `2` rotates `180` degrees and `0` for no rotation.
    """

    on_event: Optional[ft.EventHandler[ScatterChartEvent]] = None
    """
    Called when an event occurs on this chart.
    """

    def __post_init__(self, ref: Optional[ft.Ref[Any]]):
        super().__post_init__(ref)
        self._internals["skip_properties"] = ["tooltip"]



from dataclasses import field
from typing import Optional

import flet as ft

__all__ = ["BarChartRodStackItem"]


@ft.control("BarChartRodStackItem")
class BarChartRodStackItem(ft.BaseControl):
    from_y: Optional[ft.Number] = None
    """
    The starting position of this item inside a bar rod.
    """

    to_y: ft.Number = 0
    """
    The ending position of this item inside a bar rod.
    """

    color: Optional[ft.ColorValue] = None
    """
    The color of this item.
    """

    border_side: ft.BorderSide = field(default_factory=lambda: ft.BorderSide.none())
    """
    A border around this item.
    """



import asyncio
import logging
from dataclasses import dataclass, field
from io import BytesIO
from typing import Any, Optional

import flet as ft
import flet.canvas as fc

_MATPLOTLIB_IMPORT_ERROR: Optional[ImportError] = None

try:
    import matplotlib  # type: ignore[import]
    from matplotlib.figure import Figure  # type: ignore[import]
except ImportError as e:  # pragma: no cover - depends on optional dependency
    matplotlib = None  # type: ignore[assignment]
    Figure = Any  # type: ignore[assignment]
    _MATPLOTLIB_IMPORT_ERROR = e
else:
    matplotlib.use("module://flet_charts.matplotlib_backends.backend_flet_agg")

__all__ = [
    "MatplotlibChart",
    "MatplotlibChartMessageEvent",
    "MatplotlibChartToolbarButtonsUpdateEvent",
]

logger = logging.getLogger("flet-charts.matplotlib")

figure_cursors = {
    "default": None,
    "pointer": ft.MouseCursor.CLICK,
    "crosshair": ft.MouseCursor.PRECISE,
    "move": ft.MouseCursor.MOVE,
    "wait": ft.MouseCursor.WAIT,
    "ew-resize": ft.MouseCursor.RESIZE_LEFT_RIGHT,
    "ns-resize": ft.MouseCursor.RESIZE_UP_DOWN,
}


def _require_matplotlib() -> None:
    if matplotlib is None:
        raise ModuleNotFoundError(
            'Install "matplotlib" Python package to use MatplotlibChart control.'
        ) from _MATPLOTLIB_IMPORT_ERROR


@dataclass
class MatplotlibChartMessageEvent(ft.Event["MatplotlibChart"]):
    message: str
    """
    Message text.
    """


@dataclass
class MatplotlibChartToolbarButtonsUpdateEvent(ft.Event["MatplotlibChart"]):
    back_enabled: bool
    """
    Whether Back button is enabled or not.
    """
    forward_enabled: bool
    """
    Whether Forward button is enabled or not.
    """


@ft.control(kw_only=True, isolated=True)
class MatplotlibChart(ft.GestureDetector):
    """
    Displays a [Matplotlib](https://matplotlib.org/) chart.

    Warning:
        This control requires the [`matplotlib`](https://matplotlib.org/)
        Python package to be installed.

        See this [installation guide](index.md#installation) for more information.
    """

    figure: Figure = field(metadata={"skip": True})
    """
    Matplotlib figure to draw - an instance of
    [`matplotlib.figure.Figure`](https://matplotlib.org/stable/api/_as_gen/matplotlib.figure.Figure.html#matplotlib.figure.Figure).
    """

    on_message: Optional[ft.EventHandler[MatplotlibChartMessageEvent]] = None
    """
    The event is triggered on figure message update.
    """

    on_toolbar_buttons_update: Optional[
        ft.EventHandler[MatplotlibChartToolbarButtonsUpdateEvent]
    ] = None
    """
    Triggers when toolbar buttons status is updated.
    """

    def init(self):
        _require_matplotlib()
        super().init()

    def build(self):
        self.mouse_cursor = ft.MouseCursor.WAIT
        self.__started = False
        self.__dpr = self.page.media.device_pixel_ratio
        logger.debug(f"DPR: {self.__dpr}")
        self.__image_mode = "full"

        self.canvas = fc.Canvas(
            # resize_interval=10,
            on_resize=self._on_canvas_resize,
            expand=True,
        )
        self.keyboard_listener = ft.KeyboardListener(
            self.canvas,
            autofocus=True,
            on_key_down=self._on_key_down,
            on_key_up=self._on_key_up,
        )
        self.content = self.keyboard_listener
        self.on_enter = self._on_enter
        self.on_hover = self._on_hover
        self.on_exit = self._on_exit
        self.on_pan_start = self._pan_start
        self.on_pan_update = self._pan_update
        self.on_pan_end = self._pan_end
        self.on_right_pan_start = self._right_pan_start
        self.on_right_pan_update = self._right_pan_update
        self.on_right_pan_end = self._right_pan_end
        self.img_count = 1
        self._receive_queue = asyncio.Queue()
        self._main_loop = asyncio.get_event_loop()
        self._width = 0
        self._height = 0
        self._waiting = False

    def _on_key_down(self, e):
        logger.debug(f"ON KEY DOWN: {e}")

    def _on_key_up(self, e):
        logger.debug(f"ON KEY UP: {e}")

    def _on_enter(self, e: ft.HoverEvent):
        logger.debug(f"_on_enter: {e.local_position.x}, {e.local_position.y}")
        self.send_message(
            {
                "type": "figure_enter",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 0,
                "buttons": 0,
                "modifiers": [],
            }
        )

    def _on_hover(self, e: ft.HoverEvent):
        logger.debug(f"_on_hover: {e.local_position.x}, {e.local_position.y}")
        self.send_message(
            {
                "type": "motion_notify",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 0,
                "buttons": 0,
                "modifiers": [],
            }
        )

    def _on_exit(self, e: ft.HoverEvent):
        logger.debug(f"_on_exit: {e.local_position.x}, {e.local_position.y}")
        self.send_message(
            {
                "type": "figure_leave",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 0,
                "buttons": 0,
                "modifiers": [],
            }
        )

    def _pan_start(self, e: ft.DragStartEvent):
        logger.debug(f"_pan_start: {e.local_position.x}, {e.local_position.y}")
        asyncio.create_task(self.keyboard_listener.focus())
        self.send_message(
            {
                "type": "button_press",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 0,
                "buttons": 1,
                "modifiers": [],
            }
        )

    def _pan_update(self, e: ft.DragUpdateEvent):
        logger.debug(f"_pan_update: {e.local_position.x}, {e.local_position.y}")
        self.send_message(
            {
                "type": "motion_notify",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 0,
                "buttons": 1,
                "modifiers": [],
            }
        )

    def _pan_end(self, e: ft.DragEndEvent):
        logger.debug(f"_pan_end: {e.local_position.x}, {e.local_position.y}")
        self.send_message(
            {
                "type": "button_release",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 0,
                "buttons": 0,
                "modifiers": [],
            }
        )

    def _right_pan_start(self, e: ft.PointerEvent):
        logger.debug(f"_pan_start: {e.local_position.x}, {e.local_position.y}")
        self.send_message(
            {
                "type": "button_press",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 2,
                "buttons": 2,
                "modifiers": [],
            }
        )

    def _right_pan_update(self, e: ft.PointerEvent):
        logger.debug(f"_pan_update: {e.local_position.x}, {e.local_position.y}")
        self.send_message(
            {
                "type": "motion_notify",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 0,
                "buttons": 2,
                "modifiers": [],
            }
        )

    def _right_pan_end(self, e: ft.PointerEvent):
        logger.debug(f"_pan_end: {e.local_position.x}, {e.local_position.y}")
        self.send_message(
            {
                "type": "button_release",
                "x": e.local_position.x * self.__dpr,
                "y": e.local_position.y * self.__dpr,
                "button": 2,
                "buttons": 0,
                "modifiers": [],
            }
        )

    def will_unmount(self):
        """
        Called when the control is about to be removed from the page.
        """
        self.figure.canvas.manager.remove_web_socket(self)

    def home(self):
        """
        Resets the view to the original state.
        """
        logger.debug("home)")
        self.send_message({"type": "toolbar_button", "name": "home"})

    def back(self):
        """
        Goes back to the previous view.
        """
        logger.debug("back()")
        self.send_message({"type": "toolbar_button", "name": "back"})

    def forward(self):
        """
        Goes forward to the next view.
        """
        logger.debug("forward)")
        self.send_message({"type": "toolbar_button", "name": "forward"})

    def pan(self):
        """
        Activates the pan tool.
        """
        logger.debug("pan()")
        self.send_message({"type": "toolbar_button", "name": "pan"})

    def zoom(self):
        """
        Activates the zoom tool.
        """
        logger.debug("zoom()")
        self.send_message({"type": "toolbar_button", "name": "zoom"})

    def download(self, format) -> bytes:
        """
        Downloads the current figure in the specified format.
        Args:
            format (str): The format to download the figure in (e.g., 'png',
                'jpg', 'svg', etc.).
        Returns:
            bytes: The figure image in the specified format as a byte array.
        """
        logger.debug(f"Download in format: {format}")
        buff = BytesIO()
        self.figure.savefig(buff, format=format, dpi=self.figure.dpi * self.__dpr)
        return buff.getvalue()

    async def _receive_loop(self):
        while True:
            is_binary, content = await self._receive_queue.get()
            if is_binary:
                logger.debug(f"receive_binary({len(content)})")
                if self.__image_mode == "full":
                    await self.canvas.clear_capture()

                self.canvas.shapes = [
                    fc.Image(
                        src=content,
                        x=0,
                        y=0,
                        width=self.figure.bbox.size[0] / self.__dpr,
                        height=self.figure.bbox.size[1] / self.__dpr,
                    )
                ]
                ft.context.disable_auto_update()
                self.canvas.update()
                await self.canvas.capture()
                self.img_count += 1
                self._waiting = False
            else:
                logger.debug(f"receive_json({content})")
                if content["type"] == "image_mode":
                    self.__image_mode = content["mode"]
                elif content["type"] == "cursor":
                    self.mouse_cursor = figure_cursors[content["cursor"]]
                    self.update()
                elif content["type"] == "draw" and not self._waiting:
                    self._waiting = True
                    self.send_message({"type": "draw"})
                elif content["type"] == "rubberband":
                    if len(self.canvas.shapes) == 2:
                        self.canvas.shapes.pop()
                    if (
                        content["x0"] != -1
                        and content["y0"] != -1
                        and content["x1"] != -1
                        and content["y1"] != -1
                    ):
                        x0 = content["x0"] / self.__dpr
                        y0 = self._height - content["y0"] / self.__dpr
                        x1 = content["x1"] / self.__dpr
                        y1 = self._height - content["y1"] / self.__dpr
                        self.canvas.shapes.append(
                            fc.Rect(
                                x=x0,
                                y=y0,
                                width=x1 - x0,
                                height=y1 - y0,
                                paint=ft.Paint(
                                    stroke_width=1, style=ft.PaintingStyle.STROKE
                                ),
                            )
                        )
                    self.canvas.update()
                elif content["type"] == "resize":
                    self.send_message({"type": "refresh"})
                elif content["type"] == "message":
                    await self._trigger_event(
                        "message", {"message": content["message"]}
                    )
                elif content["type"] == "history_buttons":
                    await self._trigger_event(
                        "toolbar_buttons_update",
                        {
                            "back_enabled": content["Back"],
                            "forward_enabled": content["Forward"],
                        },
                    )

    def send_message(self, message):
        """Sends a message to the figure's canvas manager."""
        logger.debug(f"send_message({message})")
        manager = self.figure.canvas.manager
        if manager is not None:
            manager.handle_json(message)

    def send_json(self, content):
        """Sends a JSON message to the front end."""
        logger.debug(f"send_json: {content}")
        self._main_loop.call_soon_threadsafe(
            lambda: self._receive_queue.put_nowait((False, content))
        )

    def send_binary(self, blob):
        """Sends a binary message to the front end."""
        self._main_loop.call_soon_threadsafe(
            lambda: self._receive_queue.put_nowait((True, blob))
        )

    async def _on_canvas_resize(self, e: fc.CanvasResizeEvent):
        logger.debug(f"on_canvas_resize: {e.width}, {e.height}")

        if not self.__started:
            self.__started = True
            asyncio.create_task(self._receive_loop())
            self.figure.canvas.manager.add_web_socket(self)
            self.send_message({"type": "send_image_mode"})
            self.send_message(
                {"type": "set_device_pixel_ratio", "device_pixel_ratio": self.__dpr}
            )
            self.send_message({"type": "refresh"})
        self._width = e.width
        self._height = e.height
        self.send_message(
            {"type": "resize", "width": self._width, "height": self._height}
        )



from dataclasses import field
from typing import Optional

import flet as ft

__all__ = ["PieChartSection"]


@ft.control("PieChartSection")
class PieChartSection(ft.BaseControl):
    """
    Configures a [PieChart][(p).] section.
    """

    value: ft.Number
    """
    Determines how much the section should occupy. This depends on sum of all sections,
    each section should occupy (`value` / sum of all values) * `360` degrees.
    """

    radius: Optional[ft.Number] = None
    """
    External radius of the section.
    """

    color: Optional[ft.ColorValue] = None
    """
    Background color of the section.
    """

    border_side: ft.BorderSide = field(default_factory=lambda: ft.BorderSide.none())
    """
    The border around section shape.
    """

    title: Optional[str] = None
    """
    A title drawn at the center of the section.
    """

    title_style: Optional[ft.TextStyle] = None
    """
    The style to draw `title` with.
    """

    title_position: Optional[ft.Number] = None
    """
    The position/offset of the [`title`][(c).] relative to the section's center.

    - `0.0`: near the center
    - `1.0`: near the outside of the chart

    By default the [`title`][(c).] is drawn in the middle of the section.
    """

    badge: Optional[ft.Control] = None
    """
    An optional `Control` drawn in the middle of a section.
    """

    badge_position: Optional[ft.Number] = None
    """
    The position/offset of the [`badge`][(c).] relative to the section's center.

    - `0.0`: near the center
    - `1.0`: near the outside of the chart

    By default the [`badge`][(c).] is drawn in the middle of the section.
    """

    gradient: Optional[ft.Gradient] = None
    """
    Defines the gradient of section. If specified, overrides the color setting.
    """

    def init(self):
        super().init()
        self._internals["skip_properties"] = ["badge"]
